#include "SDL.h"
#include "SDL_ttf.h"

#include "string.h"

#include "variables.h"

#include "MOG2_auxiliar.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif


VARIABLE::VARIABLE(char *n,int v)
{
	name=new char[strlen(n)+1];
	strcpy(name,n);
	type=VALUE_INTEGER;
	ivalue=v;
	svalue=0;
	ovalue=0;
} /* VARIABLE::VARIABLE */ 


VARIABLE::VARIABLE(char *n,char *v)
{
	name=new char[strlen(n)+1];
	strcpy(name,n);
	type=VALUE_STRING;
	svalue=new char[strlen(v)+1];
	strcpy(svalue,v);
	ovalue=0;
} /* VARIABLE::VARIABLE */ 


VARIABLE::VARIABLE(char *n,ActiveOBJECT *o)
{
	name=new char[strlen(n)+1];
	strcpy(name,n);
	type=VALUE_OBJECT;
	ivalue=0;
	svalue=0;
	ovalue=o;
} /* VARIABLE::VARIABLE */ 


VARIABLE::VARIABLE(void)
{
	name=0;
	type=VALUE_INTEGER;
	ivalue=0;
	svalue=0;
	ovalue=0;
} /* VARIABLE::VARIABLE */ 


VARIABLE::VARIABLE(VARIABLE &v)
{
	name=new char[strlen(v.name)+1];
	strcpy(name,v.name);
	type=v.type;
	if (type==VALUE_STRING) {
		svalue=new char[strlen(v.svalue)+1];
		strcpy(svalue,v.svalue);
	} else {
		ivalue=v.ivalue;
		svalue=0;
	} /* if */ 
	ovalue=v.ovalue;
} /* VARIABLE::VARIABLE */ 


VARIABLE::~VARIABLE(void)
{
	delete []name;
	name=0;
	delete []svalue;
	svalue=0;
	ovalue=0;
} /* VARIABLE::~VARIABLE */ 


bool VARIABLE::save(FILE *fp)
{
	fprintf(fp,"NAME %s\n",name);
	if (type==VALUE_INTEGER) {
		fprintf(fp,"INTEGER %i\n",ivalue);
	} else if (type==VALUE_STRING) {
		char *s1;
		encode_string(svalue,&s1);
		fprintf(fp,"STRING %s\n",s1);
		delete []s1;
	} else {
		/* OBJECT REFERENCES ARE STILL NOT SUPPORTED TO SAVE!!! */ 
		/* ... */ 
	} /* if */ 
	return true;
} /* VARIABLE::save */ 


bool VARIABLE::load(FILE *fp)
{
	char tmp[80];

	if (1!=fscanf(fp,"%s ",tmp)) return false;
	fgets(tmp,80,fp);
	while((tmp[strlen(tmp)-1]==13) ||
		  (tmp[strlen(tmp)-1]==10)) tmp[strlen(tmp)-1]=0;

	if (name!=0) delete name;
	name=new char[strlen(tmp)+1];
	strcpy(name,tmp);


	if (1!=fscanf(fp,"%s ",tmp)) return false;
	if (strcmp(tmp,"INTEGER")==0) {
		type=VALUE_INTEGER;
		if (1!=fscanf(fp,"%i",&ivalue)) return false;
		delete []svalue;
		svalue=0;
	} else if (strcmp(tmp,"STRING")==0) {
		delete []svalue;
		if (!load_string(fp,&svalue)) return false;
		type=VALUE_STRING;
	} else {
		/* OBJECT REFERENCES ARE STILL NOT SUPPORTED TO LOAD!!! */ 
		/* ... */ 
	} /* if */ 
	
	return true;
} /* VARIABLE::load */ 


bool VARIABLE::cmp(char *n)
{
	if (strcmp(n,name)==0) return true;

	return false;
} /* VARIABLE::cmp */ 


int VARIABLE::get(int *ival,char **sval,ActiveOBJECT **oval)
{
	if (type==VALUE_INTEGER) {
		if (ival!=0) *ival=ivalue;
	} else if (type==VALUE_STRING) {
		if (sval!=0) *sval=svalue;
	} else {
		if (oval!=0) *oval=ovalue;
	} /* if */ 
	return type;
} /* VARIABLE::get */ 


void VARIABLE::set(int v)
{
	type=VALUE_INTEGER;
	ivalue=v;
} /* VARIABLE::set */ 


void VARIABLE::set(char *v)
{
	type=VALUE_STRING;
	delete []svalue;
	svalue=new char[strlen(v)+1];
	strcpy(svalue,v);
} /* VARIABLE::set */ 


void VARIABLE::set(ActiveOBJECT  *v)
{
	type=VALUE_OBJECT;
	ovalue=v;
} /* VARIABLE::set */ 

void VARIABLE::instance(VARIABLE *v)
{
	type=v->type;
	if (type==VALUE_STRING) {
		delete []svalue;
		svalue=new char[strlen(v->svalue)+1];
		strcpy(svalue,v->svalue);
	} else if (type==VALUE_INTEGER) {
		ivalue=v->ivalue;
	} else {
		ovalue=v->ovalue;
	} /* if */ 
} /* VARIABLE::instance */ 


char *VARIABLE::generate_string(void)
{
	char *tmp;

	if (type==VALUE_INTEGER) {
		char tmp2[16];
		sprintf(tmp2,"%i",ivalue);
		tmp=new char[strlen(tmp2)+1];
		strcpy(tmp,tmp2);
		return tmp;
	} else if (type==VALUE_STRING) {
		tmp=new char[strlen(svalue)+1];
		strcpy(tmp,svalue);
		return tmp;
	} else {
		tmp=new char[6];
		strcpy(tmp,"#OBJ#");
		return tmp;
	} /* if */ 
} /* VARIABLE::generate_string */ 


char *VARIABLE::get_name(void)
{
	return name;
} /* VARIABLE::get_name */ 
